package com.diagbot.aop;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.entity.SysTaskCron;
import com.diagbot.facade.MedCheckWorkFacade;
import com.diagbot.facade.SysTaskCronFacade;
import com.diagbot.util.HttpUtils;
import com.diagbot.util.StringUtil;
import org.apache.commons.collections4.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Map;

/**
 * @Description:运行病历每次评分后生成对应的患者类型
 * @Author: songxl
 * @time: 2022/5/19 10:18
 */

@Aspect
@Component
public class RunBehospitalInfoTypeAspect {
    @Autowired
    private MedCheckWorkFacade medCheckWorkFacade;
    @Autowired
    private SysTaskCronFacade sysTaskCronFacade;

    // 操作配置织入点 analyze_rpc
    @Pointcut("execution(public * com.diagbot.web.BehospitalInfoController.analyzeApi(..))" +
            "||execution(public * com.diagbot.web.BehospitalInfoController.analyze_rpc(..))" +
            "||execution(public * com.diagbot.web.BehospitalInfoController.analyzeRun(..))")
    public void makeTypePointCut() {
    }

    /**
     * 操作后执行
     *
     * @param joinPoint
     * @param jsonResult
     * @Return void
     */
    @AfterReturning(pointcut = "makeTypePointCut()", returning = "jsonResult")
    public void analyzeAfterReturning(JoinPoint joinPoint, Object jsonResult) {
        try {
            //获取请求入参
            String params = getRequestParams(joinPoint);
            if (StringUtils.isNotBlank(params)) {
                //生成患者类型
                makeType(params);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 存储消息通知
     *
     * @param params
     * @Return void
     */
    private void makeType(String params) {
        //参数处理
        if (StringUtil.isNotEmpty(params) && "{".equals(params.substring(0, 1))) {
            JSONObject paramJSON = JSONObject.parseObject(params);
            if (StringUtil.isBlank(paramJSON.getString("behospitalCode"))) {
                return;
            }
            SysTaskCron task = sysTaskCronFacade.getOne(new QueryWrapper<SysTaskCron>()
                    .eq("cron_code", "TASK019"), false);
            //2.执行定时任务
            if (null != task && StringUtil.isNotBlank(task.getCron())) {
                try {
                    medCheckWorkFacade.execute(task.getParam(), paramJSON.getString("behospitalCode"));
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
    }


    /**
     * /**
     * 获取请求的参数
     *
     * @param joinPoint
     * @Return java.lang.String
     */
    private String getRequestParams(JoinPoint joinPoint) throws Exception {
        Map<String, String[]> map = HttpUtils.getHttpServletRequest().getParameterMap();
        if (MapUtils.isNotEmpty(map)) {
            String params = JSONObject.toJSONString(map);
            return params;
        } else {
            Object[] args = joinPoint.getArgs();
            if (null != args) {
                return argsArrayToString(joinPoint.getArgs());
            }
        }
        return "";
    }

    /**
     * 参数拼装
     *
     * @param paramsArray
     * @Return java.lang.String
     */
    private String argsArrayToString(Object[] paramsArray) {
        String params = "";
        if (paramsArray != null && paramsArray.length > 0) {
            for (int i = 0; i < paramsArray.length; i++) {
                if (null != (paramsArray[i])) {
                    Object jsonObj = JSONObject.toJSONString(paramsArray[i]);
                    params += jsonObj.toString() + " ";
                }
            }
        }
        return params.trim();
    }
}


